// Copyright (c) 2022, the R8 project authors. Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

package com.android.tools.r8.mappingcompose;

import static org.junit.Assert.assertEquals;

import com.android.tools.r8.TestBase;
import com.android.tools.r8.TestParameters;
import com.android.tools.r8.TestParametersCollection;
import com.android.tools.r8.naming.ClassNameMapper;
import com.android.tools.r8.naming.MappingComposer;
import com.android.tools.r8.utils.StringUtils;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameter;
import org.junit.runners.Parameterized.Parameters;

@RunWith(Parameterized.class)
public class ComposeSplitRangeStartTest extends TestBase {

  @Parameter() public TestParameters parameters;

  @Parameters(name = "{0}")
  public static TestParametersCollection data() {
    return getTestParameters().withNoneRuntime().build();
  }

  private static final String mappingFoo =
      StringUtils.unixLines(
          "com.foo -> a:", "    1:5:void m():41:45 -> x", "    6:10:void m():56:60 -> x");
  private static final String mappingBar =
      StringUtils.unixLines(
          "a -> b:",
          "    11:12:void x():1:2 -> y",
          "    13:17:void x():3:7 -> y",
          "    18:24:void x():8:8 -> y",
          "    25:26:void x():9:10 -> y");
  private static final String mappingResult =
      StringUtils.unixLines(
          "com.foo -> b:",
          "    11:12:void m():41:42 -> y",
          "    13:15:void m():43:45 -> y",
          "    16:17:void m():56:57 -> y",
          "    18:24:void m():58:58 -> y",
          "    25:26:void m():59:60 -> y");

  @Test
  public void testCompose() throws Exception {
    ClassNameMapper mappingForFoo = ClassNameMapper.mapperFromString(mappingFoo);
    ClassNameMapper mappingForBar = ClassNameMapper.mapperFromString(mappingBar);
    String composed = MappingComposer.compose(mappingForFoo, mappingForBar);
    assertEquals(mappingResult, composed);
  }
}
